%%Decision Boundary Carvin Example
%Author: Filipe de Oliveira Costa
%Contact: fcosta@ic.unicamp.br
%
% Running: example (featFilename)
%
%   featFilename:
%       'diabetes.txt'
%       'german-numer.txt'
%     
%%( https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/binary.html )
%%
function example(featFilename)
FEAT = load(featFilename);

%SEPARATING POSITIVE AND NEGATIVE FEATURES
POSITIVE = FEAT((FEAT(:,1) == +1), :);
NEGATIVE = FEAT((FEAT(:,1) == -1), :);

%POSITIVE = POSITIVE(randperm(size(POSITIVE,1)),:);
%NEGATIVE = NEGATIVE(randperm(size(NEGATIVE,1)),:);

[xp, yp] = size(POSITIVE);
[xn, yn] = size(NEGATIVE);

%100 FOR TRAINING
test_positive = POSITIVE(1:xp/2 , :);
test_negative = NEGATIVE(1:xn/2 , :);

%THE REST FOR TESTING
train_positive = POSITIVE(xp/2 + 1:xp, :);
train_negative = NEGATIVE(xn/2 + 1:xn, :);

% CREATING A SVM MODEL (with linear kernel and fixed parameters, just for exemplifying)
TRAIN = cat(1, train_positive, train_negative);
vet = [' -s ' ' 0 ' ' -t ' ' 0 ' ' -c ' ' 2000 ' ' -q ' ' 1 ' ];

M = svmtrain(TRAIN(:,1), TRAIN(:,(2:yp)), vet);
%Getting the decision values in the training set
fprintf('Results using the training set...\n');
fprintf('Positive class - ');
[~, accTRP, decVPos] = svmpredict(train_positive(:, 1), train_positive(:, 2:yp), M);
fprintf('Negative class - ');
[~, accTRN, decVNeg] = svmpredict(train_negative(:, 1), train_negative(:, 2:yn), M);
%Relative Accuracy in training
AccRTR = (accTRP(1) + accTRN(1))/2;

fprintf('Relative Accuracy (on training) = %f\n',AccRTR);

fprintf('\nCalculating epsilon (Decision Boundary Carving)...\n(Using the decision values of training set results)\n');
%Decision boundary carving 
[T_1, AR_1] = decisionBoundaryCarving(decVPos, decVNeg, (AccRTR/100), 0.001, 'restrict');
[T_2, AR_2] = decisionBoundaryCarving(decVPos, decVNeg, (AccRTR/100), 0.001, 'relax');
[T_3, AR_3] = decisionBoundaryCarving(decVPos, decVNeg, (AccRTR/100), 0.001, 'average');

fprintf('e_restrict = %f (Relative accuracy on training: %f)\n', T_1, AR_1);
fprintf('e_relax = %f (Relative accuracy on training: %f)\n', T_2, AR_2);
fprintf('e_average = %f (Relative accuracy on training: %f)\n', T_3, AR_3);

%Obtaining the test results (Classifying the classes separately)
fprintf('\nResults using the test set (without DBC)...\n');
fprintf('Positive class - ');
[~, accPos, decVPos] = svmpredict(test_positive(:, 1), test_positive(:, 2:yp), M);
fprintf('Negative class - ');
[~, accNeg, decVNeg] = svmpredict(test_negative(:, 1), test_negative(:, 2:yn), M);

%Printing the accuracies 
fprintf('\n---------------\nFINAL RESULTS:\n');
%Without DBC function
fprintf('Without DBC (e = 0): (%f + %f)/2 = %f\n', accPos(1), accNeg(1), (accPos(1) + accNeg(1))/2);

%With DBC (restrict)
AccP = (sum(decVPos >= T_1)/length(decVPos));
AccN = (sum(decVNeg < T_1)/length(decVNeg));
AccR = (AccP + AccN) /2;
fprintf('With DBC (e = %f): (%f + %f)/2 = %f\n', T_1, AccP*100, AccN*100, AccR*100);

%With DBC (relax)
AccP = (sum(decVPos >= T_2)/length(decVPos));
AccN = (sum(decVNeg < T_2)/length(decVNeg));
AccR = (AccP + AccN) /2;
fprintf('With DBC (e = %f): (%f + %f)/2 = %f\n', T_2, AccP*100, AccN*100, AccR*100);

%With DBC (average)
AccP = (sum(decVPos >= T_3)/length(decVPos));
AccN = (sum(decVNeg < T_3)/length(decVNeg));
AccR = (AccP + AccN) /2;
fprintf('With DBC (e = %f): (%f + %f)/2 = %f\n', T_3, AccP*100, AccN*100, AccR*100);
fprintf('\n------------------------------\n');
end
%%




  

